import threading
import time
import serial
import conf


class PowerBox:
    """
    Interface to the Power Box v2 board.
    """
    def __init__(self, parent):
        self.parent = parent
        self.connected = False
        self.box = None
        self.busy = False
        self.lock = threading.Lock()

        if self.connect(conf.serial_port_primary):
            if self.handshake():
                self.connected = True
                self.port = conf.serial_port_primary

        if not self.connected:
            if self.connect(conf.serial_port_backup):
                if self.handshake():
                    self.connected = True
                    self.port = conf.serial_port_backup

        if not self.connected:
            exit(1)

    def _print(self, msg):
        self.parent.statusbar.SetStatusText(msg, 0)
        # print(msg)

    def reconnect(self):
        if self.connect(conf.serial_port_primary):
            if self.handshake():
                self.connected = True
                self.port = conf.serial_port_primary

        if not self.connected:
            if self.connect(conf.serial_port_backup):
                if self.handshake():
                    self.connected = True
                    self.port = conf.serial_port_backup

    def connect(self, port):
        try:
            self.box = serial.Serial(port, 230400, timeout=.25)
            return True

        except serial.serialutil.SerialException:
            return False

    def write(self, cmd):
        self.lock.acquire()
        res = b''
        try:
            self.box.write(cmd)
            res = self._readline()
        except serial.SerialException as e:
            self.parent.statusbar.SetStatusText(str(e), 0)
        finally:
            pass
        self.lock.release()
        return res

    def handshake(self):
        passcnt = 5
        rsp = ""
        cmd = bytearray("VER?\n", encoding='utf8')
        self.box.write(cmd)
        while rsp[0:2] != "OK":
            time.sleep(2)
            rsp = self._readline().decode()
            self._print("{}\t{}".format(cmd.decode()[:-1], rsp))
            passcnt -= 1
            if not passcnt:
                self._print("Port failed")
                break
        if passcnt:
            return True
        return False

    def _readline(self):
        eol = b'\n'
        line = bytearray()
        while True:
            c = self.box.read(1)
            if c:
                if c == eol:
                    break
                line += c
            else:
                break
        return line

    def get_auto_pwron(self, chnl):
        rsp = None
        if -1 < chnl < 8:
            cmd = bytearray("PWA?{}\n".format(chnl), encoding='utf8')
            brsp = self.write(cmd)
            rsp = brsp.decode()
            self._print("{}\t{}".format(cmd.decode()[:-1], rsp))
        return True if rsp[4] == 'N' else False

    def set_auto_pwron(self, chnl, on):
        rsp = None
        if -1 < chnl < 8:
            cmd = bytearray("PWA#{}:{}\n".format(chnl, on), encoding='utf8')
            brsp = self.write(cmd)
            rsp = brsp.decode()
            self._print("{}\t{}".format(cmd.decode()[:-1], rsp))
        return True if rsp[0] == 'O' else False

    def get_chnl_state(self, chnl):
        rsp = None
        if -1 < chnl < 8:
            cmd = bytearray("PWR?{}\n".format(chnl), encoding='utf8')
            brsp = self.write(cmd)
            rsp = brsp.decode()
            self._print("{}\t{}".format(cmd.decode()[:-1], rsp))
            if len(rsp) < 5:
                return False
            # OK:ON, OK:OFF, ERR:xxxx
        return True if rsp[4] == 'N' else False

    def set_chnl_state(self, chnl, state: int):
        rsp = None
        if -1 < chnl < 8:
            cmd = bytearray("PWR#{}:{}\n".format(chnl, state), encoding='utf8')
            brsp = self.write(cmd)
            rsp = brsp.decode()
            self._print("{}\t{}".format(cmd.decode()[:-1], rsp))
            # OK or ERR:xxxx
        return True if rsp[0] == 'O' else False

    def get_pwr_amps(self, chnl):
        cmd = bytearray("PWC?{}\n".format(chnl), encoding='utf8')
        brsp = self.write(cmd)
        rsp = brsp.decode()
        self._print("{}\t{}".format(cmd.decode()[:-1], rsp))
        try:
            val = float(rsp[3:9])
        except:
            val = 0.0
        return val

    def get_htr_amps(self, chnl):
        cmd = bytearray("HTC?{}\n".format(chnl), encoding='utf8')
        brsp = self.write(cmd)
        rsp = brsp.decode()
        self._print("{}\t{}".format(cmd.decode()[:-1], rsp))
        try:
            val = float(rsp[3:9])
        except:
            val = 0.0
        return val

    def get_cur_amps(self):
        cmd = bytearray("CUR?\n", encoding='utf8')
        brsp = self.write(cmd)
        rsp = brsp.decode()
        self._print("{}\t{}".format(cmd.decode()[:-1], rsp))
        try:
            val = float(rsp[3:9])
        except:
            val = 0.0
        return val

    def get_volts(self):
        cmd = bytearray("VIN?\n", encoding='utf8')
        brsp = self.write(cmd)
        rsp = brsp.decode()
        self._print("{}\t{}".format(cmd.decode()[:-1], rsp))
        try:
            val = float(rsp[3:9])
        except:
            val = 0.0
        return val

    def get_hah(self, chnl):
        cmd = bytearray("HAH?{}\n".format(chnl), encoding='utf8')
        brsp = self.write(cmd)
        rsp = brsp.decode()
        self._print("{}\t{}".format(cmd.decode()[:-1], rsp))
        try:
            val = float(rsp[3:9])
        except:
            val = 0.0
        return val

    def get_pah(self, chnl):
        cmd = bytearray("PAH?{}\n".format(chnl), encoding='utf8')
        brsp = self.write(cmd)
        rsp = brsp.decode()
        self._print("{}\t{}".format(cmd.decode()[:-1], rsp))
        try:
            val = float(rsp[3:9])
        except:
            val = 0.0
        return val

    def get_ah_total(self):
        cmd = bytearray("AHT?\n", encoding='utf8')
        brsp = self.write(cmd)
        rsp = brsp.decode()
        self._print("{}\t{}".format(cmd.decode()[:-1], rsp))
        try:
            val = float(rsp[3:9])
        except ValueError:
            val = 0.0
        return val

    def get_run_time(self):
        cmd = bytearray("RNT?\n", encoding='utf8')
        brsp = self.write(cmd)
        rsp = brsp.decode()
        self._print("{}\t{}".format(cmd.decode()[:-1], rsp))
        try:
            val = rsp[3:12]
        except:
            val = "000:00:00"
        return val

    def reset(self):

        cmd = bytearray("CLR#\n", encoding='utf8')
        brsp = self.write(cmd)
        rsp = brsp.decode()
        self._print("{}\t{}".format(cmd.decode()[:-1], rsp))
        # OK or ERR:xxxx
        return True if rsp[0] == 'O' else False
